/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2015  SEGGER Microcontroller GmbH & Co. KG        *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: Internal                                         *
*                                                                    *
*       Current version number will be inserted here                 *
*       when shipment is built.                                      *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : embOSPluginRowley.js
Purpose : Script for thread windows for embOS and Rowley CrossWorks
--------  END-OF-HEADER  ---------------------------------------------
*/

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
var TS_MASK_SUSPEND_CNT = (0x03 << 0);
var TS_MASK_TIMEOUT     = (0x01 << 2);
var TS_MASK_STATE       = (0x1f << 3);
var TS_READY            = (0 << 3);  /* ready                         */
var TS_WAIT_EVENT       = (1 << 3);  /* waiting for event             */
var TS_WAIT_SEMAZ       = (2 << 3);  /* waiting for sema zero         */
var TS_WAIT_ANY         = (3 << 3);  /* Waiting for ...  any reason   */
var TS_WAIT_SEMANZ      = (4 << 3);  /* Waiting for sema not zero     */
var TS_WAIT_MEMF        = (5 << 5);  /* Waiting for Memory pool       */
var TS_WAIT_Q           = (6 << 3);  /* Waiting for Queue             */
var TS_WAIT_MBNF        = (7 << 3);  /* Waiting for mailbox not full  */
var TS_WAIT_MBNE        = (8 << 3);  /* Waiting for mailbox not empty */
var TS_WAIT_EVENTOBJ    = (9 << 3);  /* Waiting for event object      */
var TS_EXECUTING        = (1 << 8);
var TS_TERMINATED       = (1 << 9);  /* terminated */

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/
/*********************************************************************
*
*       getregs()
*  Functions description:
*    If a thread is select in the threads window, this function
*    returns an array with the content of the register r0-r12 and
*    sp, lr, pc.
*/
function getregs(x) {
  var sp = Debug.evaluate("((OS_TASK*)"+x+")->pStack");
  var aRegs  = new Array();

  sp += 4; // skip Counters
  for (i = 4; i < 12; i++) {
    aRegs[i] = TargetInterface.peekWord(sp);
    sp+=4;
  }
  var exc_return = TargetInterface.peekWord(sp);
  sp += 4; // skip OS_REG_LR
  for (i = 0;i < 4;i++) {
    aRegs[i] = TargetInterface.peekWord(sp);
    sp +=4;
  }
  aRegs[12] = TargetInterface.peekWord(sp);
  sp += 4;
  aRegs[14] = TargetInterface.peekWord(sp);
  sp += 4;
  aRegs[15] = TargetInterface.peekWord(sp);
  sp += 4;
  aRegs[16] = TargetInterface.peekWord(sp);
  sp += 4;
  if (aRegs[16] & (1<<9)) {
    sp += 4;
  }
  if ((exc_return & (1 << 4)) == 0) {
    sp += (4*18);
  }
  aRegs[13] = sp;

  return aRegs;
}

/*********************************************************************
*
*       init()
*  Functions description:
*    This function describes the look of the threads window.
*/
function init() {
  Threads.setColumns("Priority", "Id", "Name", "State", "Timeout", "Stack max./total @ location", "Run count");
  Threads.setSortByNumber("Priority");
}

/*********************************************************************
*
*       getstate()
*  Functions description:
*    This function returns the state of a task.
*/
function getState(state) {
  var stateStr;

  if (state == TS_READY) {
    return "Ready";
  }
  if (state == TS_TERMINATED) {
    return "Terminated";
  }
  if (state == TS_MASK_TIMEOUT) {
    return "Delay";
  }
  switch (state & (TS_MASK_STATE | TS_MASK_TIMEOUT)) {
    case TS_READY:
      return "Ready";
    case TS_WAIT_EVENT:
    case TS_WAIT_EVENT | TS_MASK_TIMEOUT:
      return "Wait Task Event";
    case TS_WAIT_SEMAZ:
    case TS_WAIT_SEMAZ | TS_MASK_TIMEOUT:
      return "Wait R-Sema";
    case TS_WAIT_SEMANZ:
    case TS_WAIT_SEMANZ | TS_MASK_TIMEOUT:
      return "Wait C-Sema";
    case TS_WAIT_MBNE:
    case TS_WAIT_MBNE | TS_MASK_TIMEOUT:
      return "GetMail";
    case TS_WAIT_MBNF:
    case TS_WAIT_MBNF | TS_MASK_TIMEOUT:
      return "PutMail";
    case TS_WAIT_Q:
    case TS_WAIT_Q | TS_MASK_TIMEOUT:
      return "Get Queue";
    case TS_WAIT_ANY:
    case TS_WAIT_ANY | TS_MASK_TIMEOUT:
      return "Blocked";
    case TS_WAIT_EVENTOBJ:
    case TS_WAIT_EVENTOBJ | TS_MASK_TIMEOUT:
      return "Wait Event Obj";
    default:
      return "invalid";
  }
}

/*********************************************************************
*
*       update()
*  Functions description:
*    This function updates the thread window and its entries.
*/
function update() {
  var pCurrentTask;
  var OS_TASK;
  var pTask;
  var Id;
  var i;
  var abStack;
  var StackUsage;
  var MaxStackUsed;
  var NumActivations;

  Threads.clear();
  pTask        = Debug.evaluate("OS_Global->pTask");
  pCurrentTask = Debug.evaluate("OS_Global->pCurrentTask");
  if (pTask) {
   Threads.newqueue("Task List");
  }
  while (pTask != 0) {
    OS_TASK = Debug.evaluate("*(OS_TASK*)" + pTask);
    Id      = Debug.evaluate("(OS_TASK*)" + pTask);
    //
    // Stack/NumActivations is available only in debug builds
    //
    if (OS_TASK.StackSize) {
      abStack = TargetInterface.peekBytes(OS_TASK.pStackBot, OS_TASK.StackSize);
      for (i = 0; i < OS_TASK.StackSize; i++) {
        if (abStack[i].toString(16) != "cd") {
          break;
        }
      }
      MaxStackUsed = OS_TASK.StackSize - i;
      StackUsage   = MaxStackUsed + " / " + OS_TASK.StackSize + " @ 0x" + OS_TASK.pStackBot.toString(16).toUpperCase();
    } else {
      StackUsage = "n.a. in Release";
    }
    NumActivations = (OS_TASK.NumActivations) ? OS_TASK.NumActivations : "n.a. in Release";
    if (pTask == pCurrentTask) {
      //
      // Current task
      //
      Threads.add(OS_TASK.Priority,                      // Priority
                  "0x" + Id.toString(16).toUpperCase(),  // ID
                  OS_TASK.Name,                          // Taskname
                  "Executing",                           // Taskstate
                  OS_TASK.Timeout,                       // Timeout in ms
                  StackUsage,                            // Stack usage
                  NumActivations, []);                   // Run count
    } else {
      //
      // Suspended task
      //
      Threads.add(OS_TASK.Priority,                      // Priority
                  "0x" + Id.toString(16).toUpperCase(),  // ID
                  OS_TASK.Name,                          // Taskname
                  getState(OS_TASK.Stat),                // Taskstate
                  OS_TASK.Timeout,                       // Timeout in ms
                  StackUsage,                            // Stack usage
                  NumActivations, pTask);                // Run count
    }
    pTask = OS_TASK.pNext;
  }
}

/****** End Of File *************************************************/

