/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*     Internet: segger.com   Support: support_embos@segger.com       *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.16                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for AT91SAM7X-EK
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/

#define _AT91C_PIOA_BASE_ADDR  (0xFFFFF400)
#define _AT91C_PIOB_BASE_ADDR  (0xFFFFF600)

#define _AT91C_PMC_BASE_ADDR   (0xFFFFFC00)

#define _PIOA_ID          (2)
#define _PIOB_ID          (3)
#define _PIO_PER_OFFS     (0x00)
#define _PIO_OER_OFFS     (0x10)
#define _PIO_SODR_OFFS    (0x30)    /* Set output data              */
#define _PIO_CODR_OFFS    (0x34)    /* Clear output data register   */
#define _PIO_ODSR_OFFS    (0x38)    /* output data status register  */
#define _PIO_PDSR_OFFS    (0x3c)    /* pin data status register     */

// ========== Register definition for RSTC peripheral ==========
#define AT91C_RSTC_RSR  (*(volatile unsigned*) 	0xFFFFFD04) // (RSTC) Reset Status Register
#define AT91C_RSTC_RMR  (*(volatile unsigned*) 	0xFFFFFD08) // (RSTC) Reset Mode Register
#define AT91C_RSTC_RCR  (*(volatile unsigned*) 	0xFFFFFD00) // (RSTC) Reset Control Register

#define AT91C_RSTC_ERSTL          (0xF <<  8) // (RSTC) User Reset Length
#define AT91C_RSTC_EXTRST         (0x1 <<  3) // (RSTC) External Reset
#define AT91C_RSTC_NRSTL          (1UL << 16) // (RSTC) NRST pin level

// ========== Register definition for PIOB peripheral ==========

#define AT91C_PIOB_PPUDR (*(volatile unsigned*) 0xFFFFF660) // (PIOB) Pull-up Disable Register
#define AT91C_PIOB_PER  (*(volatile unsigned*) 	0xFFFFF600) // (PIOB) PIO Enable Register
#define AT91C_PIOB_OER  (*(volatile unsigned*) 	0xFFFFF610) // (PIOB) Output Enable Register
#define AT91C_PIOB_ODR  (*(volatile unsigned*) 	0xFFFFF614) // (PIOB) Output Disable Register
#define AT91C_PIOB_SODR (*(volatile unsigned*) 	0xFFFFF630) // (PIOB) Clear Output Data Register
#define AT91C_PIOB_CODR (*(volatile unsigned*) 	0xFFFFF634) // (PIOB) Clear Output Data Register
#define AT91C_PIOB_PDR  (*(volatile unsigned*) 	0xFFFFF604) // (PIOB) PIO Disable Register
#define AT91C_PIOB_ASR  (*(volatile unsigned*) 	0xFFFFF670) // (PIOB) Select A Register

// ========== Register definition for PMC peripheral ==========
#define AT91C_PMC_PCER  (*(volatile unsigned*) 	0xFFFFFC10) // (PMC) Peripheral Clock Enable Register

/****** SFRs and identifiers used for LED demo **********************/

#define _PMC           (*(volatile unsigned int*) _AT91C_PMC_BASE_ADDR)

/****** Assign LEDs to Ports ****************************************/

/****** Target specific configuration *******************************/

#define _NUM_LEDS      (4)
#define _LEDPORT_BASE_ADDR  (_AT91C_PIOB_BASE_ADDR)
#define _LEDPORT_ID    (_PIOB_ID)
#define _LED0_BIT      (19) /* LED1 on SAM7X-EK board, active low */
#define _LED1_BIT      (20) /* LED2 on SAM7X-EK board, active low */
#define _LED2_BIT      (21) /* LED3 on SAM7X-EK board, active low */
#define _LED3_BIT      (22) /* LED4 on SAM7X-EK board, active low */

#define _LEDPORT_PER   (*(volatile unsigned int*) (_LEDPORT_BASE_ADDR + _PIO_PER_OFFS))
#define _LEDPORT_OER   (*(volatile unsigned int*) (_LEDPORT_BASE_ADDR + _PIO_OER_OFFS))
#define _LEDPORT_SODR  (*(volatile unsigned int*) (_LEDPORT_BASE_ADDR + _PIO_SODR_OFFS))
#define _LEDPORT_CODR  (*(volatile unsigned int*) (_LEDPORT_BASE_ADDR + _PIO_CODR_OFFS))
#define _LEDPORT_STATE (*(volatile unsigned int*) (_LEDPORT_BASE_ADDR + _PIO_ODSR_OFFS))

#define _LEDMASK_ALL   ((1 << _LED0_BIT) | (1 << _LED1_BIT)) | ((1 << _LED2_BIT) | (1 << _LED3_BIT))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  _PMC |= (1 << _LEDPORT_ID);     /* Enable peripheral clock for LED-Port    */
  _LEDPORT_PER   = _LEDMASK_ALL;  /* enable Port           */
  _LEDPORT_OER   = _LEDMASK_ALL;  /* switch to output mode */
  _LEDPORT_SODR |= _LEDMASK_ALL;  /* Initially clear LEDs  */
}



/*********************************************************************
*
*       BSP_ETH_Init()
*/
void BSP_ETH_Init(unsigned Unit) {
  unsigned v;

  (void) Unit;   // Multiple units not supported by this version, suppress warning

  AT91C_PMC_PCER                 = (1 << _PIOB_ID); // Enable clock for PIOB
  AT91C_PIOB_PPUDR               = 1UL << 15;              // Disable RXDV pullup, enter PHY normal mode
  AT91C_PIOB_PPUDR               = 1UL << 16;
  //
  // Init PIO and perform a RESET of PHY since PHY
  //
  v =                            0
                                 | (1 << 0)
                                 | (1 << 15)
                                 | (1 << 16)
                                 | (1 << 18)
                                 ;

  AT91C_PIOB_PER                 = v; // Entire lower 19 bits enabled
  AT91C_PIOB_OER                 = v;
  AT91C_PIOB_CODR                = 0
                                 | (1 << 7)      // 0: node mode, 1: repeater mode
                                 | (1 << 15)     // 0: Normal mode, 1: test mode
                                 | (1 << 16)     // 0: MII
                                 | (1 << 18)     // 0: Power down
                                 ;

  AT91C_PIOB_SODR                = 0
                                 | (1 << 0)      // Isolate
                                 ;

  //
  // Perform hardware reset using RESET pin of MCU
  //
  AT91C_RSTC_RMR = (0xA5000000 | AT91C_RSTC_ERSTL) & (1 << 8);
  AT91C_RSTC_RCR =  0xA5000000 | AT91C_RSTC_EXTRST;
  while ((AT91C_RSTC_RSR & AT91C_RSTC_NRSTL) == 0);       // Wait until RESET timer has expired (just a few microseconds)
  //
  // Switch to peripheral functions
  //
  v = 0x3FFFF;                // Lower 18 bits are used for the peripheral
  AT91C_PIOB_ODR  = v;        // Entire lower 18 bits disabled
  AT91C_PIOB_ASR  = v;        // Select peripheral A use
  AT91C_PIOB_PDR  = v;        // Disable GPIO mode, select peripheral
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {
  if (Index > (_NUM_LEDS - 1)) {
    return;
  }
  _LEDPORT_CODR = (1 << (_LED0_BIT + Index));         /* Switch on LED */
}

void BSP_ClrLED(int Index) {
  if (Index > (_NUM_LEDS - 1)) {
    return;
  }
  _LEDPORT_SODR = (1 << (_LED0_BIT + Index));         /* Switch off LED */
}

void BSP_ToggleLED(int Index) {
  if (Index > (_NUM_LEDS - 1)) {
    return;
  }
  if (_LEDPORT_STATE & (1 << (_LED0_BIT + Index))) {  /* LED is toggled */
    BSP_SetLED(Index);
  } else {
    BSP_ClrLED(Index);
  }
}

/****** EOF *********************************************************/

