/*****************************************************************************
 *                   SEGGER Microcontroller GmbH & Co. KG                    *
 *            Solutions for real time microcontroller applications           *
 *****************************************************************************
 *                                                                           *
 *               (c) 2015 SEGGER Microcontroller GmbH & Co. KG               *
 *                                                                           *
 *           Internet: www.segger.com   Support: support@segger.com          *
 *                                                                           *
 *****************************************************************************/

function InitClocksAndMemories()
{
  var i;
  // Set up PLL A
  TargetInterface.pokeWord(0xFFFFFC28, 0x20603F08); // MULA=97, DIVA=8 - 16.36766 Mhz/8*97 = 198.4578775
  TargetInterface.delay(10);//while (!(TargetInterface.peekWord(0xFFFFFC68) & 2));
  // Use PLL A as processor clock and master clock divided by 2
  TargetInterface.pokeWord(0xFFFFFC30, 0x00000102);
  TargetInterface.delay(10);//while (!(TargetInterface.peekWord(0xFFFFFC68) & 8)); 

  TargetInterface.setMaximumJTAGFrequency(15000000);

  // Select 32 bit data bus
  TargetInterface.pokeWord(0xFFFFF804, 0xFFFF0000); 
  TargetInterface.pokeWord(0xFFFFF870, 0xFFFF0000);

  // assign EBI0_CSA1 to SDRAM
  TargetInterface.pokeWord(0xFFFFED20, TargetInterface.peekWord(0xFFFFED20)| 2);
  
  // SDRAM timings and configuration
  TargetInterface.pokeWord(0xFFFFE208, 0x85227259);
  TargetInterface.delay(10);
  
  // "All Banks Precharge"
  TargetInterface.pokeWord(0xFFFFE200, 2);
  TargetInterface.pokeWord(0x20000000, 0);
  TargetInterface.delay(10);

  // 8 "Auto Refresh Commands"
  for (i=0;i<8;i++)
    {
      TargetInterface.pokeWord(0xFFFFE200, 4);
      TargetInterface.pokeWord(0x20000000, 0);
    }
  // "Load Mode Register"
  TargetInterface.pokeWord(0xFFFFE200, 3);
  TargetInterface.pokeWord(0x20000000, 0);  
  // Set Refresh Rate to 7.8 us
  TargetInterface.pokeWord(0xFFFFE204, 780);  
  // Switch to normal mode
  TargetInterface.pokeWord(0xFFFFE200, 0);
  TargetInterface.pokeWord(0x20000000, 0);  
}

//load("AT91SAM9_Target.js");

function OnConnect()
{
  TargetInterface.setMaximumJTAGFrequency(5000);
}

function OnAttach()
{   
  TargetInterface.setMaximumJTAGFrequency(15000000);  
}

function OldStyleInit()
{
  var i;  
  TargetInterface.resetAndStop(100);
  TargetInterface.getICEBreakerRegister(5); /* Clear out Debug Comms Data */  
  // Make sure caches and mmu are turned off
  i = TargetInterface.executeMRC(0xEE010F00);
  TargetInterface.executeMCR(0xEE010F00, i & ~(1<<0|1<<2|1<<12)); 
  //Enable the main oscillator
  TargetInterface.pokeWord(0xFFFFFC20, 0x00000801); // CKGR_MOR = 801
  TargetInterface.delay(10);//while (!(TargetInterface.peekWord(0xFFFFFC68) & 1));
  TargetInterface.pokeWord(0xFFFFFC30, 1); // CKGR_MCKR = 1
  TargetInterface.setMaximumJTAGFrequency(50000);
}

function NewStyleInit()
{ 
  var i; 
  TargetInterface.resetAndStop(100);
  TargetInterface.getICEBreakerRegister(5); /* Clear out Debug Comms Data */  
  // Make sure caches and mmu are turned off
  i = TargetInterface.executeMRC(0xEE010F00);
  TargetInterface.executeMCR(0xEE010F00, i & ~(1<<0|1<<2|1<<12));   
}

function Reset()
{
  TargetInterface.setMaximumJTAGFrequency(5000);
  var Target = TargetInterface.getProjectProperty("Target");
  if (Target == "AT91SAM9CN11" ||  Target == "AT91SAM9CN12" || Target == "AT91SAM9G15" || Target == "AT91SAM9G25" || Target == "AT91SAM9G35" ||  Target == "AT91SAM9N12" || Target == "AT91SAM9X25" || Target == "AT91SAM9X35")
    NewStyleInit();
  else
    OldStyleInit();
  // Initialise clocks and memories
  if (InitClocksAndMemories != undefined)    
    InitClocksAndMemories();          
  // Map Internal SRAM to address zero
  rv = TargetInterface.peekWord(0x00000000);
  TargetInterface.pokeWord(0x00000000, 0x87654321);
  if (TargetInterface.peekWord(0x00000000) != 0x87654321)
    {      
      if (Target == "AT91RM92002")
        TargetInterface.pokeWord(0xFFFFFF00, 0x1);
      else if (Target == "AT91SAM9260")
        TargetInterface.pokeWord(0xFFFFEF00, 0x3);
      else if (Target == "AT91SAM9261")
        TargetInterface.pokeWord(0xFFFFEE00, 0x3);
      else if (Target == "AT91SAM9261S")
        TargetInterface.pokeWord(0xFFFFEE00, 0x3);
      else if (Target == "AT91SAM9263")
        TargetInterface.pokeWord(0xFFFFED00, 0x3);
      else if (Target == "AT91SAM9CN11")
        TargetInterface.pokeWord(0xFFFFDF00, 0x3);
      else if (Target == "AT91SAM9CN12")
        TargetInterface.pokeWord(0xFFFFDF00, 0x3);
      else if (Target == "AT91SAM9G10")
        TargetInterface.pokeWord(0xFFFFEE00, 0x3);
      else if (Target == "AT91SAM9G15")
        TargetInterface.pokeWord(0xFFFFDF00, 0x3);
      else if (Target == "AT91SAM9G20")
        TargetInterface.pokeWord(0xFFFFEF00, 0x3);
      else if (Target == "AT91SAM9G25")
        TargetInterface.pokeWord(0xFFFFDF00, 0x3);
      else if (Target == "AT91SAM9G35")
        TargetInterface.pokeWord(0xFFFFDF00, 0x3);
      else if (Target == "AT91SAM9G45")
        TargetInterface.pokeWord(0xFFFFEB00, 0x3);
      else if (Target == "AT91SAM9N12")
        TargetInterface.pokeWord(0xFFFFDF00, 0x3);
      else if (Target == "AT91SAM9R64")
        TargetInterface.pokeWord(0xFFFFEF00, 0x3);
      else if (Target == "AT91SAM9RL64")
        TargetInterface.pokeWord(0xFFFFEF00, 0x3);
      else if (Target == "AT91SAM9X25")
        TargetInterface.pokeWord(0xFFFFDF00, 0x3);
      else if (Target == "AT91SAM9X35")
        TargetInterface.pokeWord(0xFFFFDF00, 0x3);
    }
  else
    TargetInterface.pokeWord(0x00000000, rv);
}

function Reset2()
{
  TargetInterface.setMaximumJTAGFrequency(5000);  
  TargetInterface.resetAndStop(100);
  TargetInterface.getICEBreakerRegister(5); /* Clear out Debug Comms Data */
}

