/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*     Internet: segger.com   Support: support_embos@segger.com       *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.16                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for AT91SAM9263-EK
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"
#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/

#define _PIOA_ID          (2)
#define _PIOB_ID          (3)
#define _PIOCDE_ID        (4)          // Parallel I/O Controller C, D and E

#define _PIOA_BASE_ADDR   (0xfffff200)
#define _PIOB_BASE_ADDR   (0xfffff400)
#define _PIOC_BASE_ADDR   (0xfffff600)
#define _PIOD_BASE_ADDR   (0xFFFFF800)
#define _PIOE_BASE_ADDR   (0xFFFFFA00)

#define _PIO_PER_OFFS     (0x00)
#define _PIO_OER_OFFS     (0x10)
#define _PIO_SODR_OFFS    (0x30)       // Set output data
#define _PIO_CODR_OFFS    (0x34)       // Clear output data register
#define _PIO_ODSR_OFFS    (0x38)       // output data status register
#define _PIO_PDSR_OFFS    (0x3c)       // pin data status register

//
// Map port for EMAC, AT91SAM9263 has EMAC connected to port E
//
#define _EMAC_PORT_ID           (_PIOCDE_ID)
#define _EMAC_PORT_BASE_ADDR    (_PIOE_BASE_ADDR)
#define _EMAC_PORT_PPUDR        (*(volatile unsigned int*) (_EMAC_PORT_BASE_ADDR + 0x60)) // Pull up disable register
#define _EMAC_PORT_PPUER        (*(volatile unsigned int*) (_EMAC_PORT_BASE_ADDR + 0x64)) // Pull up enable register
#define _EMAC_PORT_ASR          (*(volatile unsigned int*) (_EMAC_PORT_BASE_ADDR + 0x70)) // Select peripheral function A register
#define _EMAC_PORT_BSR          (*(volatile unsigned int*) (_EMAC_PORT_BASE_ADDR + 0x74)) // Select peripheral function B register
#define _EMAC_PORT_PDR          (*(volatile unsigned int*) (_EMAC_PORT_BASE_ADDR + 0x04)) // PIO disable register

#define _PIOC_PORT_BSR          (*(volatile unsigned int*) (_PIOC_BASE_ADDR + 0x74))      // Select peripheral function B register
#define _PIOC_PORT_PPUDR        (*(volatile unsigned int*) (_PIOC_BASE_ADDR + 0x60))      // Pull up disable register
#define _PIOC_PORT_PDR          (*(volatile unsigned int*) (_PIOC_BASE_ADDR + 0x04))      // PIO disable register

#define _EMAC_PORT_RXDV_BIT     (25)
#define _EMAC_PORT_RMII_BIT     (25)

/****** MATRIX definitions ******************************************/

#define _MATRIX_BASE_ADDR  (0xFFFFEC00)
#define _MATRIX_SCFG2      (*(volatile unsigned int*) (_MATRIX_BASE_ADDR + 0x48)) // Slave configuration register 2
#define _MATRIX_PRAS2      (*(volatile unsigned int*) (_MATRIX_BASE_ADDR + 0x90)) // Priority Register A for Slave 2
#define _MATRIX_SCFG3      (*(volatile unsigned int*) (_MATRIX_BASE_ADDR + 0x4C)) // Slave configuration register 3
#define _MATRIX_PRAS3      (*(volatile unsigned int*) (_MATRIX_BASE_ADDR + 0x98)) // Priority Register A for Slave 3
#define _MATRIX_SCFG4      (*(volatile unsigned int*) (_MATRIX_BASE_ADDR + 0x50)) // Slave configuration register 4
#define _MATRIX_PRAS4      (*(volatile unsigned int*) (_MATRIX_BASE_ADDR + 0xA0)) // Priority Register A for Slave 4


/****** SFRs and identifiers used for LED demo **********************/
#define _PMC_BASE_ADDR     (0xFFFFFC00)
#define _PMC_PCER          (*(volatile unsigned int*) (_PMC_BASE_ADDR + 0x10))

/****** Assign LEDs to Ports ****************************************/

/****** Target specific configuration *******************************/

#define _LEDPORT0_BASE_ADDR  (_PIOB_BASE_ADDR)
#define _LEDPORT1_BASE_ADDR  (_PIOC_BASE_ADDR)

#define _LEDPORT0_ID         (_PIOB_ID)
#define _LEDPORT1_ID         (_PIOCDE_ID)

#define _LED0_BIT            ( 8) // LED1 on AT91SAM9263-EK board, active low
#define _LED1_BIT            (29) // LED2 on AT91SAM9263-EK board, active low

#define _LEDPORT0_PER        (*(volatile unsigned int*) (_LEDPORT0_BASE_ADDR + _PIO_PER_OFFS))
#define _LEDPORT0_OER        (*(volatile unsigned int*) (_LEDPORT0_BASE_ADDR + _PIO_OER_OFFS))
#define _LEDPORT0_SODR       (*(volatile unsigned int*) (_LEDPORT0_BASE_ADDR + _PIO_SODR_OFFS))
#define _LEDPORT0_CODR       (*(volatile unsigned int*) (_LEDPORT0_BASE_ADDR + _PIO_CODR_OFFS))
#define _LEDPORT0_STATE      (*(volatile unsigned int*) (_LEDPORT0_BASE_ADDR + _PIO_ODSR_OFFS))

#define _LEDPORT1_PER        (*(volatile unsigned int*) (_LEDPORT1_BASE_ADDR + _PIO_PER_OFFS))
#define _LEDPORT1_OER        (*(volatile unsigned int*) (_LEDPORT1_BASE_ADDR + _PIO_OER_OFFS))
#define _LEDPORT1_SODR       (*(volatile unsigned int*) (_LEDPORT1_BASE_ADDR + _PIO_SODR_OFFS))
#define _LEDPORT1_CODR       (*(volatile unsigned int*) (_LEDPORT1_BASE_ADDR + _PIO_CODR_OFFS))
#define _LEDPORT1_STATE      (*(volatile unsigned int*) (_LEDPORT1_BASE_ADDR + _PIO_ODSR_OFFS))

#define _LEDMASK0_ALL        (1 << _LED0_BIT)
#define _LEDMASK1_ALL        (1 << _LED1_BIT)

/*********************************************************************
*
*       Static const
*
**********************************************************************
*/
typedef struct {
  volatile unsigned int * pSODR;
  volatile unsigned int * pCODR;
  volatile unsigned int * pSTATE;
  char                    Id;
} LED_TYPE;

static const LED_TYPE _aLED[] = {
  {&_LEDPORT0_SODR, &_LEDPORT0_CODR, &_LEDPORT0_STATE, _LED0_BIT},
  {&_LEDPORT1_SODR, &_LEDPORT1_CODR, &_LEDPORT1_STATE, _LED1_BIT}
};

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  _PMC_PCER     |= (1 << _LEDPORT0_ID)  /* Enable peripheral clock for LED-Ports    */
                |  (1 << _LEDPORT1_ID)
                 ;
  _LEDPORT0_PER   = _LEDMASK0_ALL;       /* enable Port           */
  _LEDPORT0_OER   = _LEDMASK0_ALL;       /* switch to output mode */
  _LEDPORT0_SODR |= _LEDMASK0_ALL;       /* Initially clear LEDs  */
  _LEDPORT1_PER   = _LEDMASK1_ALL;       /* enable Port           */
  _LEDPORT1_OER   = _LEDMASK1_ALL;       /* switch to output mode */
  _LEDPORT1_SODR |= _LEDMASK1_ALL;       /* Initially clear LEDs  */
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if(Index > 1) {
    return;
  }
  *_aLED[Index].pCODR = (1 << _aLED[Index].Id);         /* Switch on LED 0  */
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index > 1) {
    return;
  }
  *_aLED[Index].pSODR = (1 << _aLED[Index].Id);         /* Switch on LED 0  */
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if(Index > 1) {
    return;
  }
  if (*_aLED[Index].pSTATE & (1 << _aLED[Index].Id)) {  /* LED is switched off */
    BSP_SetLED(Index);
  } else {
    BSP_ClrLED(Index);
  }
}

/*********************************************************************
*
*       BSP_CACHE_CleanRange()
*/
void BSP_CACHE_CleanRange (void * p, unsigned NumBytes) {
  OS_ARM_DCACHE_CleanRange(p, NumBytes);
}

/*********************************************************************
*
*       BSP_CACHE_InvalidateRange()
*/
void BSP_CACHE_InvalidateRange(void * p, unsigned NumBytes) {
  OS_ARM_DCACHE_InvalidateRange(p, NumBytes);
}

/*********************************************************************
*
*       BSP_ETH_Init()
*
*  Function description
*    This function is called from the network interface driver.
*    It initializes the network interface. This function should be used
*    to enable the ports which are connected to the network hardware.
*    It is called from the driver during the initialization process.
*
*  Note:
*    (1) If your MAC is connected to the PHY via Media Independent
*        Interface (MII) change the macro _USE_RMII and call
*        IP_NI_ConfigPHYMode() from within IP_X_Config()
*        to change the default of driver.
*
*/
void BSP_ETH_Init(unsigned Unit) {
  unsigned int PinsA_EMAC;
  unsigned int PinsB_PIOC;

  (void) Unit;    // Multiple units not supported by this version, suppress warning

  _PMC_PCER          = (1 << _EMAC_PORT_ID);        // Enable clock for PIO
  _PIOC_PORT_PPUDR   = (1 << _EMAC_PORT_RXDV_BIT);  // Disable RXDV pullup, enter PHY normal mode
#if 1
   //
   // By default driver uses RMII. Note that IP_NI_ConfigPHYMode() has also
   // to called from within IP_X_Config to switch to MII mode.
   //
  _EMAC_PORT_PPUER   = (1 << _EMAC_PORT_RMII_BIT);  // Enable Pullup  => Switch to RMII mode.
#else
  _EMAC_PORT_PPUDR   = (1 << _EMAC_PORT_RMII_BIT);  // Disable Pullup => Switch to MII mode.
#endif
  //
  // Power up PHY, may not be required, if set as hardwired option on target
  //
#ifdef _EMAC_PORT_PWR_PHY_BIT
  _EMAC_PORT_PER  = (1 << _EMAC_PORT_PWR_PHY_BIT);
  _EMAC_PORT_OER  = (1 << _EMAC_PORT_PWR_PHY_BIT);
  _EMAC_PORT_CODR = (1 << _EMAC_PORT_PWR_PHY_BIT);
#endif
  //
  // Init PIO Pins: EMAC is connected to specific lines of PIO
  //
  PinsA_EMAC           = (1uL << 21)
                       | (1uL << 23)
                       | (1uL << 24)
                       | (1uL << 25)
                       | (1uL << 26)
                       | (1uL << 27)
                       | (1uL << 28)
                       | (1uL << 29)
                       | (1uL << 30)
                       | (1uL << 31)
                       ;
  PinsB_PIOC           = (1uL << 25)
                       ;
  _EMAC_PORT_ASR = PinsA_EMAC;   // Select peripheral A use on EMAC port
  _PIOC_PORT_BSR = PinsB_PIOC;   // Select peripheral B use on PIO C
  _EMAC_PORT_PDR = PinsA_EMAC;   // Disable GPIO mode, select peripheral function
  _PIOC_PORT_PDR = PinsB_PIOC;   // Disable GPIO mode, select peripheral function
  //
  // Initialize priority of BUS MATRIX. EMAC (Master 4) needs highest priority for SDRAM on EBI0 (Slave 4)
  //
  _MATRIX_SCFG4 = 0x01120030;    // Assign EMAC as default master, activate priority arbitration, increase cycles
  _MATRIX_PRAS4 = 0x00030000;    // Set Priority of EMAC to 3 (highest value)
}

/****** End Of File *************************************************/
