/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*     Internet: segger.com   Support: support_embos@segger.com       *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.16                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit.c
          for ATMEL AT91SAM9263 CPU

Purpose : Initializes and handles the hardware for the OS as far
          as required by the OS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

#ifdef __GNUC__
  // 
  //  Suppress warning about set but unused variable "Dummy"
  //
  #pragma GCC diagnostic ignored "-Wunused-but-set-variable"
#endif

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

#define ALLOW_NESTED_INTERRUPTS 1   // Caution: Nesting interrupts will cause higher stack load on system stack CSTACK

#ifndef   _SDRAM_BASE_ADDR
  #define _SDRAM_BASE_ADDR (0x20000000)
#endif

#ifndef   SDRAM_TARGET        // Target application may run in SDRAM
  #define SDRAM_TARGET (0)    // May be overwritten by project settings
#endif

/*********************************************************************
*
*       Clock frequency settings
*/

#define OS_FSYS 200049178uL

#ifndef   OS_PCLK_TIMER
  #define OS_PCLK_TIMER (OS_FSYS / 2)
#endif

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000)
#endif

#define OS_TIMER_PRESCALE (16) // prescaler for system timer is fixed to 16

#define MUL_PLLA     (110)      // Multiplier
#define OUT_PLLA     (0x02)     // High speed setting
#define COUNT_PLLA   (0x1F)     // startup counter
#define DIV_PLLA     (9)        // Divider

#define _PLLAR_VALUE ((1uL << 29)                \
                     |((MUL_PLLA-1) << 16)       \
                     |(OUT_PLLA     << 14)       \
                     |(COUNT_PLLA   << 8)        \
                     |(DIV_PLLA     << 0))

#define MUL_PLLB     (176)      // Multiplier
#define OUT_PLLB     (0x00)     // High speed setting
#define COUNT_PLLB   (0x1f)     // startup counter
#define DIV_PLLB     (15)       // Divider
#define USB_DIV      (03)       // USB-clock Divider

#define _PLLBR_VALUE ((USB_DIV      << 28)       \
                     |((MUL_PLLB-1) << 16)       \
                     |(OUT_PLLB     << 14)       \
                     |(COUNT_PLLB   << 8)        \
                     |(DIV_PLLB     << 0))

#define MCKR_MDIV    (0x01)     // Main clock is processor clock / 2
#define MCKR_PRES    (0)        // Processor clock is selected clock
#define MCKR_CSS     (0x02)     // PLLA is selected clock

#define _MCKR_VALUE  ((MCKR_MDIV << 8)   \
                     |(MCKR_PRES << 2)   \
                     |(MCKR_CSS  << 0))

/*********************************************************************
*
*       Configuration of communication to OSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif

#ifndef   OS_VIEW_USE_UART          // If set, UART will be used
  #define OS_VIEW_USE_UART  (0)     // Default: 0 => DCC is used
#endif                              // if OS_VIEW_ENABLE is on

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently UART0 and UART1 are supported and the standard
*       setup enables UART0 per default
*       When using DCC for communiction, the UART is not used for embOSView,
*       regardless the OS_UART settings
*/

#ifndef   OS_UART
  #define OS_UART (0)
#endif

#ifndef   OS_PCLK_UART
  #define OS_PCLK_UART (OS_FSYS / 2)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400)
#endif

/****** End of configuration settings *******************************/

/*********************************************************************
*
*       Check configuration
*
**********************************************************************
*/
#ifndef   DEBUG     /* Should normally be defined as project option */
  #define DEBUG  (0)
#endif

/*********************************************************************
*
*       DCC and UART settings for OSView
*
*       Automatically generated from configuration settings
*/
#define OS_USE_DCC     (OS_VIEW_ENABLE && (OS_VIEW_USE_UART == 0))

#define OS_UART_USED   (OS_VIEW_ENABLE && (OS_VIEW_USE_UART != 0)) && ((OS_UART == 0) || (OS_UART == 1) || (OS_UART == 2))

/********************************************************************/

#if ((defined(__ICCARM__) && DEBUG) || OS_USE_DCC)
  #include "JLINKDCC.h"
#endif

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/

/*      USART, used for OSView communication */
#define _USART0_BASE_ADDR  (0xFFF8C000)
#define _USART1_BASE_ADDR  (0xFFF90000)
#define _USART2_BASE_ADDR  (0xFFF94000)

/*      Debug unit */
#define _DBGU_BASE_ADDR    (0xFFFFEE00)
#define _DBGU_IMR     (*(volatile OS_U32*) (_DBGU_BASE_ADDR + 0x10)) /* Interrupt Mask Register */
#define _DBGU_SR      (*(volatile OS_U32*) (_DBGU_BASE_ADDR + 0x14)) /* Channel Status Register */
#define DBGU_COMMRX   (1uL << 31)
#define DBGU_COMMTX   (1uL << 30)
#define DBGU_RXBUFF   (1uL << 12)
#define DBGU_TXBUFE   (1uL << 11)
#define DBGU_TXEMPTY  (1uL <<  9)
#define DBGU_PARE     (1uL <<  7)
#define DBGU_FRAME    (1uL <<  6)
#define DBGU_OVRE     (1uL <<  5)
#define DBGU_ENDTX    (1uL <<  4)
#define DBGU_ENDRX    (1uL <<  3)
#define DBGU_TXRDY    (1uL <<  1)
#define DBGU_RXRDY    (1uL <<  0)
#define DBGU_MASK_ALL (DBGU_COMMRX | DBGU_COMMTX  | DBGU_RXBUFF |  \
                       DBGU_TXBUFE | DBGU_TXEMPTY | DBGU_PARE   |  \
                       DBGU_FRAME  | DBGU_OVRE    | DBGU_ENDTX  |  \
                       DBGU_ENDRX  | DBGU_TXRDY   | DBGU_RXRDY)

/*      Reset controller */
#define _RSTC_BASE_ADDR    (0xFFFFFD00)
#define _RSTC_CR      (*(volatile OS_U32*) (_RSTC_BASE_ADDR + 0x00))
#define _RSTC_SR      (*(volatile OS_U32*) (_RSTC_BASE_ADDR + 0x04))
#define _RSTC_MR      (*(volatile OS_U32*) (_RSTC_BASE_ADDR + 0x08))
#define RSTC_URSTEN   (1uL <<  0)  /* User reset enable           */
#define RSTC_BODIEN   (1uL << 16)  /* Brownout interrupt enable   */
#define RSTC_URSTIEN  (1uL <<  4)  /* User reset interrupt enable */
#define RSTC_BODSTS   (1uL <<  1)  /* Brownout status             */
#define RSTC_URSTS    (1uL <<  0)  /* User reset status           */

/*      Real time timer */
#define _RTT_BASE_ADDR     (0xFFFFFD20)
#define _RTT_MR       (*(volatile OS_U32*) (_RTT_BASE_ADDR + 0x00))
#define _RTT_SR       (*(volatile OS_U32*) (_RTT_BASE_ADDR + 0x0C))
#define RTT_RTTINCIEN (1uL << 17)
#define RTT_ALMIEN    (1uL << 16)
#define RTT_RTTINC    (1uL << 1)
#define RTT_ALMS      (1uL << 0)

/*      Periodic interval timer */
#define _PIT_BASE_ADDR     (0xFFFFFD30)
#define _PIT_MR       (*(volatile OS_U32*) (_PIT_BASE_ADDR + 0x00))
#define _PIT_SR       (*(volatile OS_U32*) (_PIT_BASE_ADDR + 0x04))
#define _PIT_PIVR     (*(volatile OS_U32*) (_PIT_BASE_ADDR + 0x08))
#define _PIT_PIIR     (*(volatile OS_U32*) (_PIT_BASE_ADDR + 0x0C))

/*      Watchdog */
#define _WDT_BASE_ADDR     (0xFFFFFD40)
#define _WDT_CR       (*(volatile OS_U32*) (_WDT_BASE_ADDR + 0x00))
#define _WDT_MR       (*(volatile OS_U32*) (_WDT_BASE_ADDR + 0x04))
#define _WDT_SR       (*(volatile OS_U32*) (_WDT_BASE_ADDR + 0x08))
#define WDT_WDFIEN    (1uL << 12) /* Watchdog interrupt enable flag in mode register */
#define WDT_WDERR     (1uL <<  1) /* Watchdog error status flag                      */
#define WDT_WDUNF     (1uL <<  0) /* Watchdog underflow status flag                  */

/*      PIO control register */
#define _PIOA_BASE_ADDR    (0xfffff200)
#define _PIOB_BASE_ADDR    (0xfffff400)
#define _PIOC_BASE_ADDR    (0xfffff600)
#define _PIOD_BASE_ADDR    (0xfffff800)
#define _PIOE_BASE_ADDR    (0xfffffA00)

/*      PIOD, used as data BUS D16..D31 for SDRAM (EBI0) */
#define _PIOD_PDR      (*(volatile OS_U32*) (_PIOA_BASE_ADDR + 0x04))    // PIOD disable register
#define _PIOD_MDDR     (*(volatile OS_U32*) (_PIOA_BASE_ADDR + 0x54))    // PIOD multi driver disable register
#define _PIOD_ASR      (*(volatile OS_U32*) (_PIOA_BASE_ADDR + 0x70))    // PIOD peripheral A select register


/*      Power management controller */
#define _PMC_BASE_ADDR     (0xFFFFFC00)

#define _PMC_CKGR_PLLAR (*(volatile OS_U32*) (_PMC_BASE_ADDR + 0x28))  /* PLLA register */
#define _PMC_CKGR_PLLBR (*(volatile OS_U32*) (_PMC_BASE_ADDR + 0x2c))  /* PLLB register */

#define _PMC_SCDR     (*(volatile OS_U32*) (_PMC_BASE_ADDR + 0x04))  /* System Clock disable register */
#define _PMC_SCER     (*(volatile OS_U32*) (_PMC_BASE_ADDR + 0x10))  /* System clock enable register */
#define _PMC_PCER     (*(volatile OS_U32*) (_PMC_BASE_ADDR + 0x10))  /* Peripheral clock enable register */
#define _PMC_MOR      (*(volatile OS_U32*) (_PMC_BASE_ADDR + 0x20))  /* main oscillator register */
#define _PMC_PLLR     (*(volatile OS_U32*) (_PMC_BASE_ADDR + 0x2c))  /* PLL register */
#define _PMC_MCKR     (*(volatile OS_U32*) (_PMC_BASE_ADDR + 0x30))  /* Master clock register */
#define _PMC_SR       (*(volatile OS_U32*) (_PMC_BASE_ADDR + 0x68))  /* status register */
#define _PMC_IMR      (*(volatile OS_U32*) (_PMC_BASE_ADDR + 0x6C))  /* interrupt mask register */

#define _PMC_MOSCS    (1uL <<  0)
#define _CKGR_MAINRDY (1uL << 16)
#define _PMC_LOCKA    (1uL <<  1)
#define _PMC_LOCKB    (1uL <<  2)
#define _PMC_MCKRDY   (1uL <<  3)
#define _PMC_PCKRDY2  (1uL << 10)
#define _PMC_PCKRDY1  (1uL <<  9)
#define _PMC_PCKRDY0  (1uL <<  8)
#define _PMC_MASK_ALL (_PMC_PCKRDY2 | _PMC_PCKRDY1 | _PMC_PCKRDY0 | \
                       _PMC_MCKRDY  | _PMC_LOCKB    | _PMC_MOSCS)

/*      Advanced interrupt controller (AIC) */
#define _AIC_BASE_ADDR      (0xfffff000)
#define _AIC_SMR_BASE_ADDR  (_AIC_BASE_ADDR + 0x00)
#define _AIC_SVR_BASE_ADDR  (_AIC_BASE_ADDR + 0x80)
#define _AIC_SVR0      (*(volatile OS_U32*) (_AIC_SVR_BASE_ADDR + 0x00))
#define _AIC_SVR1      (*(volatile OS_U32*) (_AIC_SVR_BASE_ADDR + 0x04))
#define _AIC_IVR       (*(volatile OS_U32*) (_AIC_BASE_ADDR + 0x100))
#define _AIC_ISR       (*(volatile OS_U32*) (_AIC_BASE_ADDR + 0x108))
#define _AIC_IPR       (*(volatile OS_U32*) (_AIC_BASE_ADDR + 0x10c))
#define _AIC_IDCR      (*(volatile OS_U32*) (_AIC_BASE_ADDR + 0x124))
#define _AIC_ICCR      (*(volatile OS_U32*) (_AIC_BASE_ADDR + 0x128))
#define _AIC_IECR      (*(volatile OS_U32*) (_AIC_BASE_ADDR + 0x120))
#define _AIC_EOICR     (*(volatile OS_U32*) (_AIC_BASE_ADDR + 0x130))
#define _AIC_SPU       (*(volatile OS_U32*) (_AIC_BASE_ADDR + 0x134))
#define _AIC_DCR       (*(volatile OS_U32*) (_AIC_BASE_ADDR + 0x138))
#define _AIC_FFDR      (*(volatile OS_U32*) (_AIC_BASE_ADDR + 0x144))

/*      AIC interrupt sources and peripheral IDs        */
#define _SYSTEM_IRQ_ID  (1)   /* System IRQ ID             */
#define _PIOA_ID        (2)
#define _PIOB_ID        (3)
#define _PIOC_ID        (4)
#define _PIOD_ID        (4)
#define _PIOE_ID        (4)
#define _US0IRQ_ID      (7)   /* USART Channel 0 interrupt */
#define _US1IRQ_ID      (8)   /* USART Channel 1 interrupt */
#define _US2IRQ_ID      (9)   /* USART Channel 2 interrupt */

#ifndef   _NUM_INT_SOURCES
  #define _NUM_INT_SOURCES   (32)
#endif

#define _INT_PRIORITY_MASK (0x07)
#define _NUM_INT_PRIORITIES   (8)

/*      MATRIX + EBI interface */
#define _MATRIX_BASE_ADDR   (0xFFFFEC00)                                // MATRIX Base Address

#define _MATRIX_MCFG   (*(volatile OS_U32*) (_MATRIX_BASE_ADDR + 0x000)) // MATRIX Master configuration register
#define _MATRIX_EBICSA (*(volatile OS_U32*) (_MATRIX_BASE_ADDR + 0x120)) // MATRIX EBI Chip Select Assignment register

/*      SDRAM controller */
#define _SDRAMC_BASE_ADDR  (0xFFFFE200)   // SDRAMC Base Address
#define _SDRAMC_MR     (*(volatile OS_U32*) (_SDRAMC_BASE_ADDR + 0x00)) // (SDRAMC) SDRAM Controller Mode Register
#define _SDRAMC_TR     (*(volatile OS_U32*) (_SDRAMC_BASE_ADDR + 0x04)) // (SDRAMC) SDRAM Controller Refresh timer Register
#define _SDRAMC_CR     (*(volatile OS_U32*) (_SDRAMC_BASE_ADDR + 0x08)) // (SDRAMC) SDRAM Controller Configuration Register
#define _SDRAMC_LPR    (*(volatile OS_U32*) (_SDRAMC_BASE_ADDR + 0x10)) // (SDRAMC) SDRAM Controller Low Power Register
#define _SDRAMC_MDR    (*(volatile OS_U32*) (_SDRAMC_BASE_ADDR + 0x24)) // (SDRAMC) SDRAM Controller Memory Device Register

#define _SDRAMC_MODE_NORMAL_CMD   (0x0) // (SDRAMC) Normal Mode
#define _SDRAMC_MODE_NOP_CMD      (0x1) // (SDRAMC) Issue a All Banks Precharge Command at every access
#define _SDRAMC_MODE_PRCGALL_CMD  (0x2) // (SDRAMC) Issue a All Banks Precharge Command at every access
#define _SDRAMC_MODE_LMR_CMD      (0x3) // (SDRAMC) Issue a Load Mode Register at every access
#define _SDRAMC_MODE_RFSH_CMD     (0x4) // (SDRAMC) Issue a Refresh

/*      SMC, static memory controller */

#define _SMC_BASE_ADDR   (0xFFFFE400)                   // SMC  Base Address
#define _SMC2_BASE_ADDR  (_SMC_BASE_ADDR + (0x10 * 2))  // SMC2 Base Address
#define _SMC_SETUP2      (*(volatile OS_U32*) (_SMC2_BASE_ADDR + 0x00))
#define _SMC_PULSE2      (*(volatile OS_U32*) (_SMC2_BASE_ADDR + 0x04))
#define _SMC_CYCLE2      (*(volatile OS_U32*) (_SMC2_BASE_ADDR + 0x08))
#define _SMC_MODE2       (*(volatile OS_U32*) (_SMC2_BASE_ADDR + 0x0C))

/*********************************************************************
*
*       local data
*
**********************************************************************
*/

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       MMU and cache configuration
*
* NOTES:
*   The translation table has to be aligned on 16384 bytes boundary
*   The translation table has to be located in un-initialized memory,
*   because it is filled from startup, before segments and variables
*   are initialized.
*/
#ifdef __ICCARM__
  #pragma data_alignment=16384
  static __no_init unsigned int _TranslationTable [0x1000];
#elif defined __CC_ARM
  #pragma arm section zidata = "NO_INIT"         //   Has to be declared as separate section UNINIT in the scatter file
  __align(16384) static unsigned int _TranslationTable[0x1000];
  #pragma arm section zidata // back to default (.bss section)
#elif defined __GNUC__
  static unsigned int _TranslationTable[0x1000] __attribute__ ((aligned (16384), section (".no_init")));
#endif

#if DEBUG
static OS_U32 _SpuriousIrqCnt;
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _HandlePmcIrq(), Power management controller interrupt
*       If not used for application, this handler may be removed
*/
static void _HandlePmcIrq(void) {
#if DEBUG
  OS_U32 IrqSource;
  IrqSource  = _PMC_IMR;
  IrqSource &= (_PMC_SR & _PMC_MASK_ALL);
  if (IrqSource) {  /* PMC interrupt pending? */
    while(1);       /* Not implemented        */
  }
#endif
}

/*********************************************************************
*
*       _HandleRttIrq(), Real time timer interrupt handler
*       If not used for application, this handler may be removed
*/
static void _HandleRttIrq(void) {
#if DEBUG
  OS_U32 IrqStatus;
  OS_U32 IrqEnabled;

  IrqEnabled = _RTT_MR & (RTT_RTTINCIEN  | RTT_ALMIEN);
  IrqStatus  = _RTT_SR & (RTT_RTTINC | RTT_ALMS);
  if ((IrqStatus & RTT_RTTINC) && (IrqEnabled & RTT_RTTINCIEN )) { /* RTT inc. interrupt pending ? */
    while(1);                /* Not implemented */
  }
  if ((IrqStatus & RTT_ALMS) && (IrqEnabled & RTT_ALMIEN )) {      /* Alarm interrupt pending ? */
    while(1);                /* Not implemented */
  }
#endif
}

/*********************************************************************
*
*       _HandleDbguIrq(), Debug unit interrupt handler
*       If not used for application, this handler may be removed
*/
static void _HandleDbguIrq(void) {
#if DEBUG
  OS_U32 IrqSource;

  IrqSource  = _DBGU_IMR;
  IrqSource &= (_DBGU_SR & DBGU_MASK_ALL);
  if (IrqSource) { /* Any interrupt pending ? */
    while(1);      /* Not implemented         */
  }
#endif
}

/*********************************************************************
*
*       _HandleRstcIrq(), Reset controller interrupt handler
*       If not used for application, this handler may be removed
*/
static void _HandleRstcIrq(void) {
#if DEBUG
  OS_U32 IrqStatus;
  OS_U32 IrqEnabled;

  IrqEnabled = _RSTC_MR & (RSTC_BODIEN | RSTC_URSTIEN);
  IrqStatus  = _RSTC_SR & (RSTC_BODSTS | RSTC_URSTS);
  if ((IrqStatus & RSTC_BODSTS) && (IrqEnabled & RSTC_BODIEN )) {  /* Brownout interrupt pending ?   */
    while(1);                /* Not implemented */
  }
  if ((IrqStatus & RSTC_URSTS) && (IrqEnabled & RSTC_URSTIEN )) {  /* User reset interrupt pending ? */
    while(1);                /* Not implemented */
  }
#endif
}

/*********************************************************************
*
*       _HandleWdtIrq(), watchdog timer interrupt handler
*       If not used for application, this handler may be removed
*/
static void _HandleWdtIrq(void) {
#if DEBUG
  OS_U32 IrqStatus;

  IrqStatus = _WDT_SR & (WDT_WDERR | WDT_WDUNF);
  if (IrqStatus && (_WDT_MR & WDT_WDFIEN)) { /* Watchdog error interrupt pending ? */
    while(1);                                /* Not implemented */
  }
#endif
}

/*********************************************************************
*
*       _DefaultFiqHandler(), a dummy FIQ handler
*/
static void _DefaultFiqHandler(void) {
  while(1);
}

/*********************************************************************
*
*       _DefaultIrqHandler, a dummy IRQ handler
*
*       This handler is initially written into all AIC interrupt vectors
*       It is called, if no interrupt vector was installed for
*       specific interrupt source.
*       May be used during debugging to detect uninstalled interrupts
*/
static void _DefaultIrqHandler(void) {
  OS_U32 IrqSource;
  IrqSource = _AIC_ISR;  /* detect source of uninstalled interrupt */
  while(IrqSource == _AIC_ISR);
}

/*********************************************************************
*
*       _SpuriousIrqHandler(), a dummy spurious IRQ handler
*/
static void _SpuriousIrqHandler(void) {
#if DEBUG
  _SpuriousIrqCnt++;
#endif
}

/*********************************************************************
*
*       _OS_SystemIrqhandler()
*       the OS system interrupt, handles OS timer
*/
static void _OS_SystemIrqhandler(void) {
  volatile int Dummy;

  if (_PIT_SR & (1uL << 0)) {  /* Timer interupt pending?            */
    Dummy = _PIT_PIVR;         /* Reset interrupt pending condition  */
    OS_TICK_Handle();          /* Call OS tick handler               */
#if ((defined(__ICCARM__) && DEBUG) || OS_USE_DCC)
    JLINKDCC_Process();
#endif
  }
  /* Call to following handlers may be removed if not used by application */
  _HandlePmcIrq();
  _HandleRttIrq();
  _HandleDbguIrq();
  _HandleRstcIrq();
  _HandleWdtIrq();
}

/*********************************************************************
*
*       _InitAIC()
*
*       Initialize interupt controller by setting default vectors
*       and clearing all interrupts
*
* NOTES:
*   This function may be called from __low_level_init() and therefore
*   must not use or call any function which relies on any variable,
*   because variables are not initialized before __low_level_init()
*   is called !
*/
static void _InitAIC(void) {
  int  i;
  OS_ISR_HANDLER** papISR;

  _AIC_IDCR = 0xFFFFFFFF;                     /* Disable all interrupts     */
  _AIC_ICCR = 0xFFFFFFFF;                     /* Clear all interrupts       */
  _AIC_FFDR = 0xFFFFFFFF;                     /* Reset fast forcings        */
  _AIC_SVR0 = (int) _DefaultFiqHandler;       /* dummy FIQ handler          */
  _AIC_SPU  = (int) _SpuriousIrqHandler ;     /* dummy spurious handler     */
  papISR = (OS_ISR_HANDLER**) _AIC_SVR_BASE_ADDR;
  for (i = 1; i < _NUM_INT_SOURCES; i++)  {   /* initially set all sources  */
    *(papISR + i) = &_DefaultIrqHandler;      /* to dummy irq handler       */
  }
  for (i = 0; i < _NUM_INT_PRIORITIES; i++) {
    _AIC_EOICR = 0;                           /* Reset interrupt controller */
  }
#if DEBUG            // For debugging activate AIC protected mode
  _AIC_DCR |= 0x01;  // Enable AIC protected mode
#endif
}

/*********************************************************************
*
*       _InitFlash
*
* Function description
*   Initialize wait states and BUS interface for external Flash access
*   Needs to be called before main clock is switched to high speed
*   Depends on specific hardware and has to be implemented by user.
*
* NOTES: (1) Settings used here are for AT91SAM9263-EK eval board
*            and may have to be modified for other target board or frequency
*
*        (2) This function may be called from __low_level_init() and therefore
*            must not use or call any function which relies on any variable,
*            because variables are not initialized before __low_level_init()
*            is called !
*/
static void _InitFlash(void) {
}

/*********************************************************************
*
*       _InitClocks
*
* Function description
*   Initialize main clock and PLLs
*
* NOTES: (1) Settings used here are for AT91SAM9263-EK eval board
*            and may have to be modified for other target board or frequency
*
*        (2) This function may be called from __low_level_init() and therefore
*            must not use or call any function which relies on any variable,
*            because variables are not initialized before __low_level_init()
*            is called !
*/
static void _InitClocks(OS_U32 PLLA_Val, OS_U32 PLLB_Val, OS_U32 MCKR_Val) {
  //
  // Enable main clock OSC and wait until clock is stable
  //
  _PMC_MOR = ((64/8) << 8) + 0x01; // Startup time = 64 slow clocks, Main oscillator enable
  while ((_PMC_SR & _PMC_MOSCS) == 0) {
  }
  //
  // Initialize PLL A for Main clock
  //
  _PMC_CKGR_PLLAR = PLLA_Val;              // Setup PLL A
  //
  // Wait until PLLA locks
  //
  while((_PMC_SR & _PMC_LOCKA) == 0) {
  }
  //
  // Wait until the master clock is stable
  //
  while((_PMC_SR & _PMC_MCKRDY) == 0) {
  }
  //
  // Initialize PLL B for USB clock
  //
  _PMC_CKGR_PLLBR = PLLB_Val;         // Setup PLL B
  //
  // Wait until PLLB locks
  //
  while((_PMC_SR & _PMC_LOCKB) == 0)  {
  }
  //
  // Wait until the master clock is stable
  //
  while ((_PMC_SR & _PMC_MCKRDY) == 0) {
  }
  //
  // Select PLL A clock as clock source
  //
  _PMC_MCKR = MCKR_Val;
  //
  // Wait until the master clock is stable */
  //
  while((_PMC_SR & _PMC_MCKRDY) == 0) {
  }
}

/*********************************************************************
*
*       _InitRTT()
*
* Function description
*   Initialize Real time timer.
*
* NOTES: (1) Not used by embOS, we disable interrupts here to avoid unhandled interrupts
*            May be modified by user if RTT is required for application.
*/
static void _InitRTT(void) {
  //
  // Disable Real-Time Timer interrupts
  //
  _RTT_MR &= ~ (RTT_RTTINCIEN  | RTT_ALMIEN);
}


/*********************************************************************
*
*       __writedMemory32()
*/
#define __writeMemory32(Data, Addr, Zone)  ((*(volatile OS_U32*) (Addr)) = Data)

/*********************************************************************
*
*       __delay()
*/
static void __delay(OS_U32 Loops) {
  volatile unsigned int i;
  #define LOOP_CNT (OS_FSYS / 10000uL)
  while (Loops) {  // One loop should last 1ms
    for (i = 0; i < LOOP_CNT; i++) {
    }
    Loops--;
  }
}

/*********************************************************************
*
*       _InitSDRAM
*
* Function description
*   Initialize SDRAM controller
*
* NOTES: (1) Settings used here are for AT91SAM9263-EK eval board
*            and may have to be modified for other target board or frequency
*
*        (2) This function may be called from __low_level_init() and therefore
*            must not use or call any function which relies on any variable,
*            because variables are not initialized before __low_level_init()
*            is called !
*/
static void _InitSDRAM(void) {
  //
  // Setup bus interface for SDRAM if not already initialized
  //
  if ((_MATRIX_EBICSA & (1uL << 1)) == 0) {

    //*  Configure PIOs
    //*    AT91F_PIO_CfgPeriph( AT91C_BASE_PIOD, AT91C_PC16_D16 to AT91C_PC16_D31
    // pPio->PIO_ASR = periphAEnable; AT91C_PIOC_ASR ((AT91_REG *) 0xFFFFF870) // (PIOC) Select A Register
    // pPio->PIO_BSR = periphBEnable;AT91C_PIOC_BSR  ((AT91_REG *) 0xFFFFF874) // (PIOC) Select B Register
    // pPio->PIO_PDR = (periphAEnable | periphBEnable); // Set in Periph mode

    //-I- Configure PIOD as peripheral (D16/D31)

    __writeMemory32(0xFFFF0000,0xFFFFF870,"Memory");
    __writeMemory32(0x00000000,0xFFFFF874,"Memory");
    __writeMemory32(0xFFFF0000,0xFFFFF804,"Memory");

    // Init MATRIX to support EBI0 CS1 for SDRAM"
    // AT91PS_CCFG ((AT91_REG *)     0xFFFFED20)
    // (CCFG)  EBI0 Chip Select Assignement Register
    __writeMemory32(0x10002,0xFFFFED20,"Memory");

    //* psdrc->SDRAMC_CR =  AT91C_SDRAMC_NC_9  | AT91C_SDRAMC_NR_13 | AT91C_SDRAMC_CAS_2 |
    //  AT91C_SDRAMC_NB_4_BANKS | AT91C_SDRAMC_DBW_32_BITS | AT91C_SDRAMC_TWR_2 | AT91C_SDRAMC_TRC_7 |
    //  AT91C_SDRAMC_TRP_2 | AT91C_SDRAMC_TRCD_2 | AT91C_SDRAMC_TRAS_5 | AT91C_SDRAMC_TXSR_8 ;
    __writeMemory32(0x85227259,0xFFFFE208,"Memory");
    __delay(1);

    //*    psdrc->SDRAMC_MR    = 0x00000002;        // Set PRCHG AL
    __writeMemory32(0x00000002,0xFFFFE200,"Memory");

    //*    *AT91C_SDRAM    = 0x00000000;            // Perform PRCHG
    __writeMemory32(0x00000000,0x20000000,"Memory");
    __delay(1);

    //*    psdrc->AT91C_SDRAMC0_MR    = AT91C_SDRAMC_MODE_RFSH_CMD;       // Set 1st CBR
    __writeMemory32(0x00000004,0xFFFFE200,"Memory");
    //*    *(AT91C_SDRAM+4)    = 0x00000001;    // Perform CBR
    __writeMemory32(0x00000001,0x20000000,"Memory");

    //*    psdrc->AT91C_SDRAMC0_MR    = 0x00000004;                       // Set 2 CBR
    __writeMemory32(0x00000004,0xFFFFE200,"Memory");
    //*    *(AT91C_SDRAM+8)    = 0x00000002;    // Perform CBR
    __writeMemory32(0x00000000,0x20000000,"Memory");

    //*    psdrc->AT91C_SDRAMC0_MR    = AT91C_SDRAMC_MODE_RFSH_CMD;       // Set 3 CBR
    __writeMemory32(0x00000004,0xFFFFE200,"Memory");
    //*    *(AT91C_SDRAM+0xc)    = 0x00000003;    // Perform CBR
    __writeMemory32(0x00000000,0x20000000,"Memory");

    //*    psdrc->AT91C_SDRAMC0_MR    = AT91C_SDRAMC_MODE_RFSH_CMD;       // Set 4 CBR
    __writeMemory32(0x00000004,0xFFFFE200,"Memory");
    //*    *(AT91C_SDRAM+0x10)    = 0x00000004;    // Perform CBR
    __writeMemory32(0x00000000,0x20000000,"Memory");

    //*    psdrc->AT91C_SDRAMC0_MR    = AT91C_SDRAMC_MODE_RFSH_CMD;       // Set 5 CBR
    __writeMemory32(0x00000004,0xFFFFE200,"Memory");
    //*    *(AT91C_SDRAM+0x14)    = 0x00000005;    // Perform CBR
    __writeMemory32(0x00000000,0x20000000,"Memory");

    //*    psdrc->AT91C_SDRAMC0_MR    = AT91C_SDRAMC_MODE_RFSH_CMD;       // Set 6 CBR
    __writeMemory32(0x00000004,0xFFFFE200,"Memory");
    //*    *(AT91C_SDRAM+0x18)    = 0x00000006;    // Perform CBR
    __writeMemory32(0x00000000,0x20000000,"Memory");

    //*    psdrc->AT91C_SDRAMC0_MR    = AT91C_SDRAMC_MODE_RFSH_CMD;       // Set 7 CBR
    __writeMemory32(0x00000004,0xFFFFE200,"Memory");
    //*    *(AT91C_SDRAM+0x1c)    = 0x00000007;    // Perform CBR
    __writeMemory32(0x00000000,0x20000000,"Memory");

    //*    psdrc->AT91C_SDRAMC0_MR    = AT91C_SDRAMC_MODE_RFSH_CMD;       // Set 8 CBR
    __writeMemory32(0x00000004,0xFFFFE200,"Memory");
    //*    *(AT91C_SDRAM+0x20)    = 0x00000008;    // Perform CBR
    __writeMemory32(0x00000000,0x20000000,"Memory");

    //*    psdrc->AT91C_SDRAMC0_MR    = AT91C_SDRAMC_MODE_LMR_CMD;        // Set LMR operation
    __writeMemory32(0x00000003,0xFFFFE200,"Memory");
    //*    *(AT91C_SDRAM+0x20)    = 0xcafedede;        // Perform LMR burst=1, lat=2
    __writeMemory32(0xcafedede,0x20000020,"Memory");

    // Setup refresh timer. We assume one refresh cycle every 7.81 micro seconds.
    // If SDRAM runs with other refresh rates (for example 15.61 us), this may be changed
    //
    _SDRAMC_TR     = ((781 * (OS_FSYS / 1000000)) / 100);
    //*    psdrc->AT91C_SDRAMC0_MR    = AT91C_SDRAMC_MODE_NORMAL_CMD;     // Set Normal mode
    __writeMemory32(0x00000000,0xFFFFE200,"Memory");

    //*    *AT91C_SDRAM    = 0x00000000;    // Perform Normal mode
    __writeMemory32(0x00000000,0x20000000,"Memory");

  }
}

/*********************************************************************
*
*       _InitSMC
*
* Function description
*   Initialize static memory controller
*
* NOTES: (1) Settings used here are for AT91SAM9263-EK eval board
*            and may have to be modified for other target board or frequency
*
*        (2) This function may be called from __low_level_init() and therefore
*            must not use or call any function which relies on any variable,
*            because variables are not initialized before __low_level_init()
*            is called !
*/
static void _InitSMC(void) {
  #define   NRD_PULSE_2     (3)
  #define   NWR_PULSE_2     (3)
  #define   NRD_SETUP_2     (1)
  #define   NWR_SETUP_2     (1)
  #define   NCS_RD_SETUP_2  (0)
  #define   NCS_WR_SETUP_2  (0)
  #define   NCS_RD_PULSE_2  (NRD_SETUP_2 + NRD_PULSE_2 + 1 - NCS_RD_SETUP_2)
  #define   NCS_WR_PULSE_2  (NWR_SETUP_2 + NWR_PULSE_2 + 1 - NCS_WR_SETUP_2)
  #define   NRD_HOLD_2      (8)
  #define   NWR_HOLD_2      (9)
  _SMC_SETUP2 = (NCS_RD_SETUP_2 << 24)
              | (NRD_SETUP_2 << 16)
              | (NCS_WR_SETUP_2 << 8)
              | (NWR_SETUP_2 << 0)
              ;

  _SMC_PULSE2 = (NCS_RD_PULSE_2 << 24)
              | (NRD_PULSE_2 << 16)
              | (NCS_WR_PULSE_2 << 8)
              | (NWR_PULSE_2 << 0)
              ;

  _SMC_CYCLE2 = ((NRD_SETUP_2 + NRD_PULSE_2 + NRD_HOLD_2) << 16)  // NRD_SETUP + NRD_PULSE + NRD_HOLD
              | ((NWR_SETUP_2 + NWR_PULSE_2 + NWR_HOLD_2) << 0)   // NWR_SETUP + WWR_PULSE + NWR_HOLD
              ;
  _SMC_MODE2  = 0x0000
              | (1uL << 12)      // 16bit bus width
              | (1uL << 8)       // Enable BAT mode for write access
              | (0uL << 4)       // Disable NWAIT mode (not connected)
              | (1uL << 1)       // Set WRITE_MODE for write cycles (write controlled)
              | (1uL << 0)       // Set READ_MODE for read cycles (read controlled)
              ;
}

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
* Function description
*   Returns the current hardware timer count value
*
* Return value
*   Current timer count value
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return _PIT_PIIR;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
* Function description
*   Returns if the hardware timer interrupt pending flag is set
*
* Return value
*   == 0; Interrupt pending flag not set
*   != 0: Interrupt pending flag set
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return (_PIT_PIIR & 0xFFF00000);
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
#define OS_TIMER_RELOAD ((OS_PCLK_TIMER/OS_TIMER_PRESCALE/OS_TICK_FREQ) - 1)
#if (OS_TIMER_RELOAD >= 0x00100000)
  #error "PIT timer can not be used, please check configuration"
#endif

void OS_InitHW(void) {
  //
  // Structure with information about timer frequency, tick frequency, etc.
  // for micro second precise system time.
  // SysTimerConfig.TimerFreq will be set later, thus it is initialized with zero.
  //
  OS_SYSTIMER_CONFIG SysTimerConfig = {OS_PCLK_TIMER, OS_TICK_FREQ, 0, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

  OS_IncDI();
  OS_ARM_CACHE_Sync();          // Ensure, caches are synchronized
  /* Initialize PIT as OS timer, enable timer + timer interrupt */
  _PIT_MR = ((OS_TIMER_RELOAD & 0x000FFFFF) | (1uL << 25) | (1uL << 24));
  OS_ARM_InstallISRHandler(_SYSTEM_IRQ_ID, _OS_SystemIrqhandler);
  OS_ARM_EnableISR(_SYSTEM_IRQ_ID);

  //
  // Setup values for usec precise system time functions
  //
  OS_Config_SysTimer(&SysTimerConfig);

  OS_COM_Init();
  OS_DecRI();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggling an output or incrementing a counter)
*/
void OS_Idle(void) {  /* Idle loop: No task is ready to exec */
  while (1) {
#if DEBUG == 0
    _PMC_SCDR = 1;    // Switch off CPU clock to save power
#endif
  }
}

/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for task-info via embOSView or high
*       resolution time measurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt;
  OS_U32 time ;

  t_cnt = _PIT_PIIR;           /* Read current timer value   */
  time  = OS_GetTime32();      /* Read current OS time       */
  if (t_cnt & 0xFFF00000) {    /* Timer Interrupt pending ?  */
    time  += (t_cnt >> 20);    /* Adjust result              */
    t_cnt &= 0x000FFFFF;
  }
  return (OS_TIMER_RELOAD * time) + t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagnostics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  Cycles *= OS_TIMER_PRESCALE;   /* eliminate effect of prescaler */
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       Communication for OSView via UART (optional)
*
**********************************************************************
*/
#if OS_UART_USED
  #if (OS_UART == 0)
    #define _PIO_US_BASE_ADDR     (_PIOA_BASE_ADDR)
    #define _OS_UART_BASE_ADDR    (_USART0_BASE_ADDR)
    #define OS_UART_ID            (_US0IRQ_ID)
    #define OS_PIO_UART_ID        (_PIOA_ID)
    #define OS_UART_RX_PIN        (26)
    #define OS_UART_TX_PIN        (27)
  #elif (OS_UART == 1)
    #define _PIO_US_BASE_ADDR     (_PIOD_BASE_ADDR)
    #define _OS_UART_BASE_ADDR    (_USART1_BASE_ADDR)
    #define OS_UART_ID            (_US1IRQ_ID)
    #define OS_PIO_UART_ID        (_PIOD_ID)
    #define OS_UART_RX_PIN        (1)
    #define OS_UART_TX_PIN        (0)
  #elif (OS_UART == 2)
    #define _PIO_US_BASE_ADDR     (_PIOD_BASE_ADDR)
    #define _OS_UART_BASE_ADDR    (_USART2_BASE_ADDR)
    #define OS_UART_ID            (_US2IRQ_ID)
    #define OS_PIO_UART_ID        (_PIOD_ID)
    #define OS_UART_RX_PIN        (2)
    #define OS_UART_TX_PIN        (3)
  #endif

  #define _PIO_US_PDR  (*(volatile OS_U32*)(_PIO_US_BASE_ADDR + 0x04)) // PIO disable register
  #define _PIO_US_ASR  (*(volatile OS_U32*)(_PIO_US_BASE_ADDR + 0x70)) // PIO peripheral "A" select register
  #define _PIO_US_BSR  (*(volatile OS_U32*)(_PIO_US_BASE_ADDR + 0x74)) // PIO peripheral "B" select register

  #define PIO_PDR_USART ((1uL << OS_UART_RX_PIN) | (1uL << OS_UART_TX_PIN))

  #define _US_CR   (*(volatile OS_U32*)(_OS_UART_BASE_ADDR + 0x00))
  #define _US_MR   (*(volatile OS_U32*)(_OS_UART_BASE_ADDR + 0x04))
  #define _US_IER  (*(volatile OS_U32*)(_OS_UART_BASE_ADDR + 0x08))
  #define _US_IDR  (*(volatile OS_U32*)(_OS_UART_BASE_ADDR + 0x0c))
  #define _US_IMR  (*(volatile OS_U32*)(_OS_UART_BASE_ADDR + 0x10))
  #define _US_CSR  (*(volatile OS_U32*)(_OS_UART_BASE_ADDR + 0x14))
  #define _US_RHR  (*(volatile OS_U32*)(_OS_UART_BASE_ADDR + 0x18))
  #define _US_THR  (*(volatile OS_U32*)(_OS_UART_BASE_ADDR + 0x1c))
  #define _US_BRGR (*(volatile OS_U32*)(_OS_UART_BASE_ADDR + 0x20))
  #define _US_RTOR (*(volatile OS_U32*)(_OS_UART_BASE_ADDR + 0x24))
  #define _US_TTGR (*(volatile OS_U32*)(_OS_UART_BASE_ADDR + 0x28))

  #define _US_TPR  (*(volatile OS_U32*)(_OS_UART_BASE_ADDR + 0x108))
  #define _US_TCR  (*(volatile OS_U32*)(_OS_UART_BASE_ADDR + 0x10c))
  #define _US_PTCR (*(volatile OS_U32*)(_OS_UART_BASE_ADDR + 0x120))

  #define US_RXRDY   (1uL << 0)         /* Rx status flag                  */
  #define US_TXRDY   (1uL << 1)         /* Tx RDY Status flag              */
  #define US_TXEMPTY (1uL << 9)         /* Tx EMPTY Status flag            */
  #define US_ENDTX   (1uL << 4)         /* Tx end flag                     */
  #define USART_RX_ERROR_FLAGS (0xE0)   /* Parity, framing, overrun error  */

  #define OS_UART_PRIO    (0x00)        /* Lowest priority for UART interrupts */
  #define US_TX_INT_FLAG  (US_TXRDY)

/*********************************************************************
*
*       OS_COM_ISR_Usart() OS USART interrupt handler
*       handles both, Rx and Tx interrupt
*/
static void OS_COM_ISR_Usart(void) {
  int UsartStatus;
  volatile int Dummy;

  UsartStatus = _US_CSR;                   /* Examine status register */
  do {
    if (UsartStatus & US_RXRDY) {          /* Data received?          */
      if (UsartStatus & USART_RX_ERROR_FLAGS) {  /* Any error ?       */
        Dummy = _US_RHR;                   /* => Discard data         */
        _US_CR = (1uL << 8);               /* RSTSTA: Reset Status Bits PARE, FRAME, OVRE and RXBRK */
      } else {
        OS_OnRx(_US_RHR);                  /* Process actual byte     */
      }
    }
    if (UsartStatus & _US_IMR & US_TX_INT_FLAG) { /* Check Tx status => Send next character */
      if (OS_OnTx()) {                     /* No more characters to send ?  */
        _US_IDR = US_TX_INT_FLAG;          /* Disable further tx interrupts */
      }
    }
    UsartStatus = _US_CSR;                 /* Examine current status  */
  } while (UsartStatus & _US_IMR & (US_TX_INT_FLAG | US_RXRDY));
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Never call this function directly from your application
*/
void OS_COM_Send1(OS_U8 c) {
  while((_US_CSR & US_TX_INT_FLAG) == 0) { // Wait until THR becomes available
  }
  _US_THR = c;
  _US_IER  = US_TX_INT_FLAG; /* enable Tx interrupt */
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART for OSView
*/
void OS_COM_Init(void) {
  OS_IncDI();
  _PMC_PCER  = (1uL << OS_UART_ID)     /* Enable peripheral clock for selected USART */
             | (1uL << OS_PIO_UART_ID) /* Enable the PIO port where UART pins are located */
             ;
  _PIO_US_PDR  = PIO_PDR_USART;        /* Enable peripheral output signals on PIO for USART (disable PIO Port) */
  _PIO_US_ASR  = PIO_PDR_USART;        /* Select "A" peripherals on PIO for USART (USART Rx, TX) */
  _PIO_US_BSR  = 0;                    /* Deselct "B" peripherals on PIO for USART */
  _US_CR   = (1uL <<  2) |             /* RSTRX: Reset Receiver: 1 = The receiver logic is reset. */
             (1uL <<  3)               /* RSTTX: Reset Transmitter: 1 = The transmitter logic is reset. */
           ;
  _US_CR   = (0uL <<  2) |             /* RSTRX: Reset Receiver: 1 = The receiver logic is reset. */
             (0uL <<  3) |             /* RSTTX: Reset Transmitter: 1 = The transmitter logic is reset. */
             (1uL <<  4) |             /* RXEN: Receiver Enable: 1 = The receiver is enabled if RXDIS is 0. */
             (0uL <<  5) |             /* RXDIS: Receiver Disable: 0 = Noeffect. */
             (1uL <<  6) |             /* TXEN: Transmitter Enable: 1 = The transmitter is enabled if TXDIS is 0. */
             (0uL <<  7) |             /* TXDIS: Transmitter Disable: 0 = Noeffect. */
             (1uL <<  8) |             /* RSTSTA: Reset Status Bits: 1 = Resets the status bits PARE, FRAME, OVRE and RXBRK in the US_CSR. */
             (0uL <<  9) |             /* STTBRK: Start Break: 0 = Noeffect. */
             (0uL << 10) |             /* STPBRK: Stop Break: 0 = Noeffect. */
             (0uL << 11) |             /* STTTO: Start Time-out: 0 = Noeffect. */
             (0uL << 12)               /* SENDA: Send Address: 0 = Noeffect. */
           ;
  _US_MR   = (0uL <<  4) |             /* USCLKS: Clock Selection: 0 = MCK */
             (3uL <<  6) |             /* CHRL: Character Length: 3 = Eight bits */
             (0uL <<  8) |             /* SYNC: Synchronous Mode Select: 0 = USART operates in Asynchronous Mode. */
           (0x4uL <<  9) |             /* PAR: Parity Type: 0x4 = No parity */
             (0uL << 12) |             /* NBSTOP: Number of Stop Bits: 0 = 1 stop bit */
             (0uL << 14) |             /* CHMODE: Channel Mode: 0 = Normal mode */
             (0uL << 17) |             /* MODE9: 9-bit Character Length: 0 = CHRL defines character length. */
             (0uL << 18)               /* CKLO: Clock Output Select: 0 = The USART does not drive the SCK pin. */
           ;
  _US_BRGR = (OS_PCLK_UART / (OS_BAUDRATE * 16));
  _US_IDR  = 0xFFFFFFFFuL;             /* Disable all interrupts     */
  _US_IER  = (1uL << 0) |              /* Enable Rx Interrupt        */
             (0 << 1);                 /* Do not Enable Tx Interrupt */
  /* Setup interrupt controller for OS USART */
  OS_ARM_InstallISRHandler(OS_UART_ID, &OS_COM_ISR_Usart);  /* OS UART interrupt handler vector */
  OS_ARM_ISRSetPrio(OS_UART_ID, OS_UART_PRIO);              /* Level sensitive, selected priority. */
  _AIC_ICCR = (1uL << OS_UART_ID);                          /* Clear OS usart interrupt.        */
  OS_ARM_EnableISR(OS_UART_ID);                             /* Enable OS usart interrupts       */
  OS_DecRI();
}

#elif OS_USE_DCC

/*********************************************************************
*
*       _DCC_OnRx()
*/
static void _DCC_OnRx(unsigned Channel, OS_U8 Data) {
  (void) Channel;   // Not supported by this version, suppress warning
  OS_OnRx(Data);
}

/*********************************************************************
*
*       _DCC_OnTx()
*/
static void _DCC_OnTx(unsigned Channel) {
  (void) Channel;   // Not supported by this version, suppress warning
  OS_OnTx();
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize DCC for OSView
*/
void OS_COM_Init(void) {
  JLINKDCC_SetpfOnRx(1, _DCC_OnRx);
  JLINKDCC_SetpfOnTx(1, _DCC_OnTx);
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Send 1 character via DCC
*/
void OS_COM_Send1(OS_U8 c) {
  JLINKDCC_SendCharOnChannelNB(1, c);
}

#else  /* DCC not selected, selected UART not supported, using dummies */

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* avoid compiler warning */
  OS_COM_ClearTxActive();  /* let the OS know that Tx is not busy */
}

#endif /*  OS_UART_USED  */

/****** Final check of configuration ********************************/

#ifndef OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

/*********************************************************************
*
*       OS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_irq_handler
*
*       Detect reason for IRQ and call correspondig service routine.
*       OS_irq_handler is called from OS_IRQ_SERVICE function
*       found in RTOSVect.asm
*/
OS_INTERWORK void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;
  pISR = (OS_ISR_HANDLER*) _AIC_IVR;   // Read interrupt vector to release NIRQ to CPU core
#if DEBUG
  _AIC_IVR = (OS_U32) pISR;            // Write back any value to IVR register to allow interrupt stacking in protected mode
#endif
#if ALLOW_NESTED_INTERRUPTS
  OS_EnterNestableInterrupt();         // Now interrupts may be reenabled. If nesting should be allowed
#else
  OS_EnterInterrupt();                 // Inform OS that interrupt handler is running
#endif
  pISR();                              // Call interrupt service routine
  OS_DI();                             // Disable interrupts and unlock
  _AIC_EOICR = 0;                      // interrupt controller =>  Restore previous priority
#if ALLOW_NESTED_INTERRUPTS
  OS_LeaveNestableInterrupt();         // Leave nestable interrupt, perform task switch if required
#else
  OS_LeaveInterrupt();                 // Leave interrupt, perform task switch if required
#endif
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler (int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;
  OS_ISR_HANDLER** papISR;

#if OS_DEBUG
  if ((unsigned)ISRIndex >= _NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
    return NULL;
  }
#endif
  OS_DI();
  papISR = (OS_ISR_HANDLER**)_AIC_SVR_BASE_ADDR;
  pOldHandler          = *(papISR + ISRIndex);
  *(papISR + ISRIndex) = pISRHandler;
  OS_RestoreI();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_EnableISR
*/
void OS_ARM_EnableISR(int ISRIndex) {
#if OS_DEBUG
  if ((unsigned)ISRIndex >= _NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
    return;
  }
#endif

  OS_DI();
  _AIC_IECR = (1uL << ISRIndex);
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_ARM_DisableISR
*/
void OS_ARM_DisableISR(int ISRIndex) {
#if OS_DEBUG
  if ((unsigned)ISRIndex >= _NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
    return;
  }
#endif

  OS_DI();
  _AIC_IDCR = (1uL << ISRIndex);
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U32* pPrio;
  int     OldPrio;

#if OS_DEBUG
  if ((unsigned)ISRIndex >= _NUM_INT_SOURCES) {
    OS_Error(OS_ERR_ISR_INDEX);
    return 0;
  }
#endif
  OS_DI();
  pPrio = (OS_U32*)_AIC_SMR_BASE_ADDR;
  OldPrio = pPrio[ISRIndex];
  pPrio[ISRIndex] = (OldPrio & ~_INT_PRIORITY_MASK) | (Prio & _INT_PRIORITY_MASK);
  OS_RestoreI();
  return OldPrio & _INT_PRIORITY_MASK;
}

/*********************************************************************
*
*       _CopyToSDRAM
*
* Function description
*   Copy whole application from Flash to SDRAM, which has to be initialized before
*
* NOTES: (1) Settings used here are for AT91SAM9263 eval board
*            and may have to be modified for other target boards
*
*        (2) This function may be called from __low_level_init() and therefore
*            must not use or call any function which relies on any variable,
*            because variables are not initialized before __low_level_init()
*            is called !
*
*        (3) The function has to be called, before MMU is initialized and
*            SDRAM is remapped to 0x0.
*/
#if (SDRAM_TARGET == 1)
#if defined (__ICCARM__)
#if (__VER__ >= 500)
static void _CopyToSDRAM(void) {
  unsigned long Size;
  #pragma segment="PROGRAM_DATA"
  #pragma section=".intvec"
  Size = (unsigned long) __segment_end("PROGRAM_DATA") - (unsigned long) __segment_begin(".intvec");
  memcpy((void*)_SDRAM_BASE_ADDR, (void*) __segment_begin(".intvec"), Size);
}
#else
static void _CopyToSDRAM(void) {
  unsigned long Size;
  #pragma segment="PROGRAM_END"
  #pragma segment="INTVEC"
  Size = (unsigned long) __segment_begin("PROGRAM_END") - (unsigned long) __segment_begin("INTVEC");
  memcpy((void*)_SDRAM_BASE_ADDR, (void*) __segment_begin("INTVEC"), Size);
}
#endif
#endif  // __ICCARM__
#ifdef __GNUC__
static void _CopyToSDRAM(void) {
}
#endif
#endif  // (SDRAM_TARGET == 1)

/*********************************************************************
*
*       __low_level_init()
*
*       Initialize memory controller, clock generation and pll
*
*       Has to be modified, if another CPU clock frequency should be
*       used. This function is called during startup and
*       has to return 1 to perform segment initialization
*/
#ifdef __cplusplus
extern "C" {
#endif
OS_INTERWORK int __low_level_init(void);  // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
OS_INTERWORK int __low_level_init(void) {
  _WDT_MR = (1uL << 15);                    // Initially disable watchdog
  //return 1;
  //
  // Setup waitstates for external BUS interface before switching to high speed
  //
  _InitFlash();
  //
  // Initialize main clock oscillator and PLLs
  //
  _InitClocks(_PLLAR_VALUE, _PLLBR_VALUE, _MCKR_VALUE);
  //
  // Initialize real time timer
  //
  _InitRTT();
  //
  // Initialize SDRAM
  //
  _InitSDRAM();
  //
  // Copy program to SDRAM
  //
#if SDRAM_TARGET
  _CopyToSDRAM();
#endif
  //
  //  Initialize static memory controller
  //
  _InitSMC();
  //
  // Init MMU and caches. This defines the virtual memory map, which is used during execution.
  // Memory mapping should be complete, meaning 4096 entries.
  // Code below fills in ascending VAddr order
  //
  //                                             Mode                      VAddr  PAddr  Size[MB]
#if SDRAM_TARGET
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,     0x000, 0x200, 0x001);    // SDRAM, remapped
#else
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,     0x000, 0x003, 0x001);    // Internal SRAM2, remapped
#endif
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_ILLEGAL, 0x001, 0x0,   0x002);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB,   0x003, 0x003, 0x001);    // Internal SRAM2, original address
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_ILLEGAL, 0x004, 0x0,   0x001);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB,   0x005, 0x005, 0x001);    // Internal SRAM0, original address
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_ILLEGAL, 0x006, 0x0,   0x001);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB,   0x007, 0x007, 0x001);    // LCD controller
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_ILLEGAL, 0x008, 0x0,   0x002);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB,   0x00A, 0x00A, 0x001);    // USB host controller OHCI
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_ILLEGAL, 0x00B, 0x0,   0x1F5);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_C_B,     0x200, 0x200, 0x020);    // SDRAM, first 32MB
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB,   0x220, 0x220, 0x020);    // SDRAM, second 32MB
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_ILLEGAL, 0x240, 0x0,   0x0C0);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB,   0x300, 0x300, 0x001);    // External SRAM area CS2, used for Ethernet controller
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_ILLEGAL, 0x301, 0x0,   0x0FF);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB,   0x400, 0x400, 0x020);    // NAND controller
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_ILLEGAL, 0x420, 0x0,   0xBDF);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(&_TranslationTable[0], OS_ARM_CACHEMODE_NC_NB,   0xfff, 0xfff, 0x001);    // SFR area
  OS_ARM_MMU_Enable      (&_TranslationTable[0]);
  OS_ARM_ICACHE_Enable();
  OS_ARM_DCACHE_Enable();
  //
  // Initialize interrupt controller
  //
  _InitAIC();
  //
  //  Perform other initialization here, if required
  //
  _RSTC_MR = ((0xA5uL << 24) | RSTC_URSTEN);  // write KEY and URSTEN to allow USER RESET
  return 1;
}

/*****  End of file  ************************************************/


