/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*     Internet: segger.com   Support: support_embos@segger.com       *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.16                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP interface for Embedded Artists LPC2468 eval board.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"
#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define MAC_MODULEID  *(volatile unsigned int*)(0xFFE00FFC) // MAC ID register

// Pin Connect Block Registers
#define PINSEL_BASE_ADDR	0xE002C000
#define PINSEL2       *(volatile unsigned long *)(PINSEL_BASE_ADDR + 0x08)
#define PINSEL3       *(volatile unsigned long *)(PINSEL_BASE_ADDR + 0x0C)

#define INT_INDEX_MAC  21

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
}

/*********************************************************************
*
*       ETH_Init
*/
void BSP_ETH_Init(unsigned Unit) {
  (void)Unit;
  /*------------------------------------------------------------------------------
  * write to PINSEL2/3 to select the PHY functions on P1[17:0]
  *-----------------------------------------------------------------------------*/
  /* P1.6, ENET-TX_CLK, has to be set for EMAC to address a BUG in
  the rev"xx-X" or "xx-Y" silicon(see errata). On the new rev.(xxAY, released
  on 06/22/2007), P1.6 should NOT be set. */
  if (MAC_MODULEID == 0x39022000) {     // Older chip ?
    PINSEL2 = 0x50151105;	/* selects P1[0,1,4,6,8,9,10,14,15] */
  } else {
    PINSEL2 = 0x50150105;	/* selects P1[0,1,4,8,9,10,14,15] */
  }
  PINSEL3 = (PINSEL3 & ~0x0000000f) | 0x5;
}


/*********************************************************************
*
*       BSP_ETH_InstallISR
*/
void BSP_ETH_InstallISR(void (*pfISR)(void)) {
  OS_ARM_InstallISRHandler(INT_INDEX_MAC, pfISR);     // Timer/counter interrupt vector.
  OS_ARM_EnableISR(INT_INDEX_MAC);                    // Enable timer/counter 0 interrupt.
  OS_ARM_ISRSetPrio(INT_INDEX_MAC, 15);               // lowest priority
}

/*********************************************************************
*
*       LED switching routines
*/

void BSP_SetLED(int Index) {
  (void)Index;
}

void BSP_ClrLED(int Index) {
  (void)Index;
}

void BSP_ToggleLED(int Index) {
  (void)Index;
}

/****** EOF *********************************************************/

